package cn.fastfun.controller.api;

import cn.fastfun.controller.dto.DevicePageStatDTO;
import cn.fastfun.controller.param.DeviceBathQueryParam;
import cn.fastfun.controller.param.DeviceFormParam;
import cn.fastfun.controller.param.DeviceQueryParam;
import cn.fastfun.service.AppDeviceService;
import cn.fastfun.service.SysExcelFieldService;
import cn.fastfun.service.entity.AppDevice;
import cn.fastfun.util.DateUtils;
import cn.fastfun.util.ObjectUtil;
import cn.fastfun.util.VerifyUtil;
import com.bridge.dto.ApiDTO;
import com.bridge.dto.ApiPageDTO;
import com.bridge.dto.IdParam;
import com.bridge.dto.QueryParamExp;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.data.domain.Page;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Bridge AutoGen
 */
@Slf4j
@Api(tags = {"2.0 设备信息"})
@RestController
@RequestMapping("/api/v1/appdevice")
public class ApiAppDeviceController {
    //业务类
    @Resource(name = "appDeviceService")
    AppDeviceService appDeviceService;

    @Resource
    SysExcelFieldService sysExcelFieldService;

    /**
     * 数据保存
     */
    @ApiOperation(value = "单个电池保存")
    @RequestMapping(value = "save", method = RequestMethod.POST)
    public ApiDTO save(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) DeviceFormParam param) {
        // 此处定义的sn实际上是指的表中的imei，也就是实体类的属性imei
        String checkSnResult = VerifyUtil.checkSn(param.getImei());
        List<AppDevice> checkDuplicatedResultOnImei = appDeviceService.findAll(
                QueryParamExp.eq("imei", param.getImei()));
        if (!checkDuplicatedResultOnImei.isEmpty()) {
            return ApiDTO.error(500, "该imei已经被录入", checkDuplicatedResultOnImei);
        }
        List<AppDevice> checkDuplicatedResultOnSn = appDeviceService.findAll(
                QueryParamExp.eq("sn", param.getSn()));
        if (!checkDuplicatedResultOnSn.isEmpty()) {
            return ApiDTO.error(500, "该sn已经被录入", checkDuplicatedResultOnSn);
        }
        if (checkSnResult.equals("passed")) {
            AppDevice entity = new AppDevice().fromFormParam(param);// 实体
            appDeviceService.save(entity);
            return ApiDTO.ok("保存成功");
        } else {
            return ApiDTO.error(500, checkSnResult);
        }
    }


    @ApiOperation(value = "查询信息")
    @RequestMapping(value = "get", method = RequestMethod.POST)
    public ApiDTO get(@RequestBody @ApiParam(name = "批次编号", required = true) IdParam param) {
        List<AppDevice> appDevicesOnOneBatch = appDeviceService.findAll(
                QueryParamExp.eq("batchNum", param.getId()));
        return ApiDTO.ok("查询成功！", appDevicesOnOneBatch);
    }

    @ApiOperation(value = "设备分页搜索")
    @RequestMapping(value = "pageQuery", method = RequestMethod.POST)
    public ApiPageDTO pageQuery(@RequestBody DeviceQueryParam param) {
        Page<AppDevice> devicePage = appDeviceService.findByParam(param);
        devicePage.getContent().forEach(p -> appDeviceService.addSnTitle(p));
        return new ApiPageDTO(null, devicePage);
    }



    @ApiOperation(value = "导入模板下载")
    @GetMapping("excelTemplate")
    public void excelTemplate(HttpServletResponse response, @RequestParam String name) {
        HSSFWorkbook workbook = sysExcelFieldService.getTemplate("invoice");
        sysExcelFieldService.outExcelFile(workbook, response, name);
    }

    @ApiOperation(value = "设备数据导入")
    @RequestMapping(value = "excelImport", method = RequestMethod.POST)
    public ApiDTO excelImport(MultipartFile file) {
        appDeviceService.importDevice(sysExcelFieldService.importExcel("invoice", file));
        return ApiDTO.ok();
    }

    @ApiOperation(value = "入库状态统计")
    @RequestMapping(value = "count", method = RequestMethod.POST)
    public ApiDTO deviceCountStat() {
        DevicePageStatDTO dto = new DevicePageStatDTO();
        Long deviceCount = appDeviceService.getCount();
        Long inStorage = appDeviceService.getCount(QueryParamExp.eq("status", 0));
        dto.setTotal(deviceCount);
        dto.setNotInStorage(deviceCount - inStorage);
        return ApiDTO.ok(dto);
    }

    @ApiOperation(value = "设备批次分页搜索列表")
    @RequestMapping(value = "batchNum/pageQuery", method = RequestMethod.POST)
    public ApiPageDTO batchNumPageQuery(@RequestBody DeviceBathQueryParam param) {
        //@ToDo: 添加各种状态
        StringBuffer sql = new StringBuffer();
        sql.append("select t1.batch_num,t1.sn,t1.deliver_time,count(0) as total,count(if(status = 1,true,null)) as in_storage,count(if(status = 2,true,null)) as transfer,");
        sql.append("count(if(status = 3,true,null)) as out_storage,count(if(status = 4,true,null)) as handle,");
        sql.append("t1.add_time,t1.received_place from app_device t1");

        if (!StringUtils.isEmpty(param.getBatchNum())) {
            sql.append(" where t1.batch_num like '%").append(param.getBatchNum()).append("%'");
        }

        sql.append(" group by t1.batch_num");
        if (!StringUtils.isEmpty(param.getOrderBy())) {
            sql.append(param.getOrderBy());
        }
        log.info("SQL: {}", sql.toString());


        ApiPageDTO page = appDeviceService.getListBySQL(sql.toString(), new HashMap<>(), param);
        if(null != page.getData()) {
            // 组装数据
            List<Map<String, Object>> list = (List<Map<String, Object>>) page.getData();
            for (Map<String, Object> p : list) {
                AppDevice device = new AppDevice();
                device.setSn(ObjectUtil.obj2String(p.get("sn")));
                appDeviceService.addSnTitle(device);

                p.put("type_title", device.getTypeTitle());
                p.put("pack_title", device.getPackTitle());
                p.put("deliver_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("deliver_time")), DateUtils.YMD));
                p.put("add_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("add_time")), DateUtils.YMDHMS));
                p.put("operator", "admin");
            }
            page.setData(list);
        }
        return page;
    }

}