package cn.fastfun.service.impl;

import cn.fastfun.controller.param.HandelFormParam;
import cn.fastfun.controller.param.LibraryInFormParam;
import cn.fastfun.controller.param.LibraryOutFormParam;
import cn.fastfun.controller.param.TransferFormParam;
import cn.fastfun.service.AppDeviceLogService;
import cn.fastfun.service.AppDeviceService;
import cn.fastfun.service.entity.AppDevice;
import cn.fastfun.service.entity.AppDeviceLog;
import cn.fastfun.service.entity.AppImeiHistory;
import cn.fastfun.service.entity.SysUserRole;
import com.bridge.dto.QueryParamExp;
import com.bridge.exception.ApiRuntimeException;
import com.bridge.service.JpaService;
import com.bridge.service.impl.JpaServiceImp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@Slf4j
@Service
public class AppDeviceLogServiceImpl extends JpaServiceImp<AppDeviceLog, String> implements AppDeviceLogService {

    @Resource
    AppDeviceService appDeviceService;

    //业务类
    @Resource(name = "appImeiHistoryService")
    JpaService<AppImeiHistory, String> appImeiHistoryService;

    @Override
    @Transactional
    public void putInStorage(LibraryInFormParam param) {
        if (StringUtils.isEmpty(param.getSn()) && !StringUtils.isEmpty(param.getBatchNum())) {
            List<AppDevice> deviceList = appDeviceService.findAll(QueryParamExp.in("batchNum", param.getBatchNum().toArray(new String[]{})));
            param.setSn(new ArrayList<>());
            deviceList.forEach(p -> param.getSn().add(p.getSn()));
        }
        if (!CollectionUtils.isEmpty(param.getSn())) {
            Date time = new Date();
            String batchNum = UUID.randomUUID().toString();
            param.getSn().forEach(p -> {
                AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
                if (null == appDevice) throw new ApiRuntimeException("设备信息不存在!");
                if (appDevice.getStatus() <= 1) {
                    appDevice.setStatus(1); // 已入库
                    appDeviceService.save(appDevice);
                    save(new AppDeviceLog(p, appDevice.getImei()).toInStorage(time, batchNum)); // 记录入库日志
                }
            });
        }
    }

    @Override
    @Transactional
    public void transfer(TransferFormParam param) {
        if (StringUtils.isEmpty(param.getSn()) && !StringUtils.isEmpty(param.getBatchNum())) {
            List<AppDevice> deviceList = appDeviceService.findAll(QueryParamExp.in("batchNum", param.getBatchNum().toArray(new String[]{})));
            param.setSn(new ArrayList<>());
            deviceList.forEach(p -> param.getSn().add(p.getSn()));
        }
        if (!CollectionUtils.isEmpty(param.getSn())) {
            Date time = new Date();
            String batchNum = UUID.randomUUID().toString();
            param.getSn().forEach(p -> {
                AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
                if (null == appDevice) throw new ApiRuntimeException("设备信息不存在!");
                if (1 != appDevice.getStatus() && 3 != appDevice.getStatus()) throw new ApiRuntimeException("设备不能调拨!");
                appDevice.setStatus(2); // 已划拨
                appDevice.setOwnerId(param.getCustomId()); // 设置归属
                appDeviceService.save(appDevice);
                save(new AppDeviceLog(p, appDevice.getImei()).toTransfer(param, time, batchNum)); // 记录划拨日志
            });
        }
    }

    @Override
    @Transactional
    public void outStorage(LibraryOutFormParam param) {
        Date time = new Date();
        String batchNum = UUID.randomUUID().toString();
        param.getSn().forEach(p -> {
            AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
            if (null == appDevice) throw new ApiRuntimeException("设备信息不存在!");
            if (2 != appDevice.getStatus()) throw new ApiRuntimeException("设备不是划拨状态!");
            appDevice.setStatus(3); // 已出库
            appDeviceService.save(appDevice);
            save(new AppDeviceLog(p, appDevice.getImei()).outStorage(param, time, batchNum)); // 记录出库日志
        });
    }

    @Override
    @Transactional
    public void handle(HandelFormParam param) {
        Date time = new Date();
        String batchNum = UUID.randomUUID().toString();
        param.getSn().forEach(p -> {
            AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", p));
            if (null == appDevice) throw new ApiRuntimeException("设备信息不存在!");
            if (1 != appDevice.getStatus()) throw new ApiRuntimeException("设备不是入库状态!");
            appDevice.setStatus(4); // 处置
            appDeviceService.save(appDevice);
            save(new AppDeviceLog(p, appDevice.getImei()).toHandle(param, time, batchNum)); // 记录处置日志
        });
    }
}
